
#include "rtc.h"

//RTS timer bound vars
static __IO uint16_t delay_counter;

void delay_ms(uint16_t delay)
{
	delay_counter = (delay/10);
	while(delay_counter > 0);
}

void set_timeout(uint16_t timeout)
{
	delay_counter = (timeout/10);
}

uint8_t timeout_occured()
{
	if(delay_counter > 0){
		return 0;
	}else{
		return 1;
	}
}

void init_rtc(void)
/*
{
	#ifndef GLOBAL_INIT_STRUCTURES
	NVIC_InitTypeDef NVIC_InitStructure; 
	EXTI_InitTypeDef EXTI_InitStructure;
	#endif
	// Enable the PWR clock 
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

	// Allow access to RTC 
	PWR_RTCAccessCmd(ENABLE);

	// LSI used as RTC source clock 
	// The RTC Clock may varies due to LSI frequency dispersion. 
	// Enable the LSI OSC 
	RCC_LSICmd(ENABLE);

	// Wait till LSI is ready
	while(RCC_GetFlagStatus(RCC_FLAG_LSIRDY) == RESET);
	
	// Select the RTC Clock Source
	RCC_RTCCLKConfig(RCC_RTCCLKSource_LSI);

	// Enable the RTC Clock
	RCC_RTCCLKCmd(ENABLE);

	// Wait for RTC APB registers synchronisation
	RTC_WaitForSynchro();

	// EXTI configuration ******************************************************* /
	EXTI_ClearITPendingBit(EXTI_Line20);
	EXTI_InitStructure.EXTI_Line = EXTI_Line20;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);
	
	// Enable the RTC Wakeup Interrupt
	NVIC_InitStructure.NVIC_IRQChannel = RTC_WKUP_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);  

	// Configure the RTC WakeUp Clock source: CK_SPRE (1Hz)
	RTC_WakeUpClockConfig(RTC_WakeUpClock_RTCCLK_Div16);
	RTC_SetWakeUpCounter(24); //10 ms wakeup

	// Enable the RTC Wakeup Interrupt
	RTC_ITConfig(RTC_IT_WUT, ENABLE);

	// Enable Wakeup Counter
	RTC_WakeUpCmd(ENABLE);

}
*/
{
	
	RCC->APB1ENR |= RCC_APB1ENR_PWREN;
	PWR->CR |= PWR_CR_DBP; //Disable write protection
	
	RCC->CSR |= RCC_CSR_LSION; //Enable LSI
	while(!(RCC->CSR & RCC_CSR_LSIRDY)); //Wait for LSI
	
	RCC->CSR &= ~RCC_CSR_RTCSEL;
	RCC->CSR |= (RCC_CSR_RTCSEL_1 | RCC_CSR_RTCEN); //LSI as RTC clock, enable RTC
	//RCC->CSR |= RCC_CSR_RTCEN; //Enable RTC
	
	
	RTC->WPR = 0xCA;
	RTC->WPR = 0x53; //Disable the write protection for RTC register
	RTC->ISR &= ~RTC_ISR_RSF;
	while(!(RTC->ISR & RTC_ISR_RSF)); //Wait for synchro, I think
	
	//RTC->CR &= ~RTC_CR_WUTE; //Disable RTC wakeup
	//while(!(RTC->ISR & RTC_ISR_WUTWF));
	
	EXTI->PR |= EXTI_PR_PR20;
	EXTI->IMR |= EXTI_IMR_MR20;
	EXTI->EMR &= ~EXTI_EMR_MR20;
	EXTI->RTSR |= EXTI_RTSR_TR20;
	EXTI->FTSR &= ~EXTI_FTSR_TR20; //Configure EXTI As rising edge Interrupt mode
	
	NVIC->IP[RTC_WKUP_IRQn] = 0x4; //Priority 4
	// >> 0x05 take top 3 bits = 32 bits per register, &0x1f - 32 bit mask
    NVIC->ISER[RTC_WKUP_IRQn >> 0x05] = 0x01 << (RTC_WKUP_IRQn & 0x1F); //Enable interrupt - can assing due to it being positive edge triggered
    //NVIC->ICER[RTC_WKUP_IRQn >> 0x05] = 0x01 << (RTC_WKUP_IRQn & 0x1F); //Disable interrupt
	
	
	RTC->CR &= ~RTC_CR_WUCKSEL; //Set wakeup to CLK/16
	RTC->WUTR = 24; //10 ms wakeup
	
	//Enable interrupt
	RTC->CR |= RTC_CR_WUTIE;
	RTC->CR |= RTC_CR_WUTE; //Enable RTC wakeup
	
	
	
	RTC->WPR = 0xFF; //Enable write protection
	//PWR->CR &= ~PWR_CR_DBP; //Disable write protection
}

void decrement_delay(void)
{
	if(delay_counter > 0){
		delay_counter--;
	}
}

void RTC_WKUP_IRQHandler(void)
{
	if(RTC->ISR & RTC_ISR_WUTF){
		decrement_delay();
		
		RTC->ISR &= ~RTC_ISR_WUTF;
		EXTI->PR |= EXTI_PR_PR20;
	}
}